//--------------------------------------------------------------------- 
#include "MainWindow.h" 
//--------------------------------------------------------------------- 
BEditorWindow::BEditorWindow(BRect frame,const char *title) 
    :BWindow(frame,title,MAINWINDOW_WINDOWSTYLE,0) 
{ 
    mainview=new BEditorView(Bounds());    
    AddChild(mainview); 
    save_filepanel=new BFilePanel(B_SAVE_PANEL); 
    editingfile=NULL; 
} 
//--------------------------------------------------------------------- 
BEditorWindow::~BEditorWindow() 
{ 
    delete save_filepanel; 
    if(editingfile!=NULL) 
        delete editingfile; 
} 
//--------------------------------------------------------------------- 
void BEditorWindow::MessageReceived(BMessage *msg) 
{ 
    switch(msg->what) 
    { 
        case B_UNDO: 
        case B_COPY: 
        case B_CUT: 
        case B_PASTE: 
        case B_SELECT_ALL: 
            ((BMemoView *)FindView("memo"))->MessageReceived(msg); 
            break; 
        case MSG_SAVE: 
            if(editingfile!=NULL) 
            { 
                SaveFile(editingfile); 
                break; 
            } 
        case MSG_SAVEAS: 
            save_filepanel->SetTarget(this); 
            save_filepanel->Show(); 
            break; 
        case B_SAVE_REQUESTED: 
            { 
                entry_ref ref; 
                BString nam; 
                BDirectory dir; 
                BEntry entry; 

                msg->FindRef("directory",&ref); 
                msg->FindString("name",&nam); 
                dir.SetTo(&ref); 

                if(entry.SetTo(&dir,nam.String())==B_OK) 
                { 
                    SaveFile(&entry); 
                    return; 
                } 
            } 
            break;    
        default: 
            BWindow::MessageReceived(msg); 
    } 
} 
//--------------------------------------------------------------------- 
BEditorView::BEditorView(BRect frame) 
    :BView(frame,"beditorview",B_FOLLOW_ALL,B_WILL_DRAW) 
{ 
    BRect viewrect(Bounds()); 
    BMenuBar *mainmenu=new BMenuBar(BRect(0,0,viewrect.right, 
                                    be_plain_font->Size()*2), 
                                    "mainmenubar"); 
    AddChild(mainmenu); 

    memo=new BMemoView(BRect(0,be_plain_font->Size()*2, 
                             viewrect.right-B_V_SCROLL_BAR_WIDTH, 
                             viewrect.bottom-B_H_SCROLL_BAR_HEIGHT), 
                       "memo",B_FOLLOW_ALL); 
    BScrollView *scr=new BScrollView("memoscroll",memo, 
                                     B_FOLLOW_ALL,0,true,true); 
    AddChild(scr); 

    BMenu *filemenu=new BMenu("File"); 
    filemenu->AddItem(new BMenuItem("Open...",new BMessage(MSG_OPEN))); 
    filemenu->ItemAt(0)->SetTarget(be_app); 
    filemenu->AddItem(new BMenuItem("Save",new BMessage(MSG_SAVE))); 
    filemenu->AddItem(new BMenuItem("Save As...",new BMessage(MSG_SAVEAS))); 
    filemenu->AddSeparatorItem(); 
    filemenu->AddItem(new BMenuItem("Quit",new BMessage(B_QUIT_REQUESTED))); 
    mainmenu->AddItem(filemenu); 

    BMenu *editmenu=new BMenu("Edit"); 
    editmenu->AddItem(new BMenuItem("Undo",new BMessage(B_UNDO))); 
    editmenu->AddSeparatorItem(); 
    editmenu->AddItem(new BMenuItem("Copy",new BMessage(B_COPY))); 
    editmenu->AddItem(new BMenuItem("Cut",new BMessage(B_CUT))); 
    editmenu->AddItem(new BMenuItem("Paste",new BMessage(B_PASTE))); 
    editmenu->AddItem(new BMenuItem("Select All",new BMessage(B_SELECT_ALL))); 
    mainmenu->AddItem(editmenu); 

    memo->SetDoesUndo(true); 
} 
//--------------------------------------------------------------------- 
void BEditorWindow::OpenFile(BEntry *entry) 
{ 
    if(entry->Exists()) 
    { 
        BFile fil; 
        off_t size; 
    
        try 
        { 
            fil.SetTo(entry,B_READ_ONLY); 
            fil.GetSize(&size); 
            ((BTextView *)FindView("memo"))->SetText(&fil,0,size); 
            SetFileName(entry); 
        } 
        catch(...) 
        { 
            BAlert *altmsg = new BAlert("Error", 
                                        "File read failed!!","OK", 
                                        NULL,NULL,B_WIDTH_AS_USUAL, 
                                        B_WARNING_ALERT); 
            altmsg->Go(); 
        } 
    } 
} 
//--------------------------------------------------------------------- 
void BEditorWindow::SetFileName(BEntry *entry) 
{ 
    if(editingfile==NULL) 
        editingfile=new BEntry(); 
    *editingfile=*entry; 
    
    if(editingfile!=NULL) 
    { 
        char nam[B_FILE_NAME_LENGTH]; 
        
        editingfile->GetName(nam); 
        SetTitle(nam); 
    } 
} 
//--------------------------------------------------------------------- 
void BEditorWindow::SaveFile(BEntry *entry) 
{ 
    BFile fil; 
    off_t wsize; 
    off_t tsize; 

    try 
    { 
        fil.SetTo(entry,B_READ_WRITE | B_CREATE_FILE); 
        tsize=strlen(((BTextView *)FindView("memo"))->Text()); 
        wsize=fil.WriteAt(0,((BTextView *)FindView("memo"))->Text(), 
                          tsize); 
        
        if(tsize==wsize) 
            SetFileName(entry); 
    } 
    catch(...) 
    { 
        BAlert *altmsg = new BAlert("Error", "File write failed!!","OK", 
                                    NULL,NULL,B_WIDTH_AS_USUAL, 
                                    B_WARNING_ALERT); 
        altmsg->Go(); 
    } 
} 
//--------------------------------------------------------------------- 